from django import forms
from django.utils.translation import ugettext_lazy as _

from . import widgets
from .models import Jingle, Track


def get_nonstop_zones():
    from emissions.models import Nonstop

    return [(x.id, x.title) for x in Nonstop.objects.all()]


def get_optional_nonstop_zones():
    return [('', '')] + get_nonstop_zones()


def get_search_nonstop_zones():
    return (
        [('', 'All'), ('any', _('Any')), ('', '--------------')]
        + get_nonstop_zones()
        + [('', '--------------'), ('none', 'None')]
    )


def get_jingle_choices():
    return [(x.id, x.label) for x in Jingle.objects.filter(emission__isnull=True, clock_time__isnull=True)]


def get_optional_jingle_choices():
    return [('', '')] + get_jingle_choices()


class UploadTracksForm(forms.Form):
    tracks = forms.FileField(widget=forms.ClearableFileInput(attrs={'multiple': True, 'accept': 'audio/*'}))
    nonstop_zone = forms.ChoiceField(choices=get_optional_nonstop_zones)


class TrackMetaForm(forms.ModelForm):
    class Meta:
        model = Track
        fields = ['language', 'instru', 'sabam', 'cfwb', 'nonstop_zones']
        widgets = {
            'nonstop_zones': widgets.MulticheckWidget,
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['nonstop_zones'].queryset = self.fields['nonstop_zones'].queryset.order_by('start')


class TrackSearchForm(forms.Form):
    q = forms.CharField(label=_('Text'), required=False)
    zone = forms.ChoiceField(label=_('Nonstop Zone'), choices=get_search_nonstop_zones, required=False)
    order_by = forms.ChoiceField(
        label=_('Order'),
        required=False,
        choices=[
            ('title', _('Alphabetically')),
            ('-added_to_nonstop_timestamp', _('Newest first')),
            ('added_to_nonstop_timestamp', _('Oldest first')),
        ],
    )


class CleanupForm(forms.Form):
    zone = forms.ChoiceField(label=_('Nonstop Zone'), choices=get_optional_nonstop_zones)
    no_metadata = forms.BooleanField(label=_('Limit to tracks without metadata'), required=False)


class ZoneSettingsForm(forms.Form):
    start = forms.TimeField(label=_('Start'), widget=widgets.TimeWidget)
    end = forms.TimeField(label=_('End'), widget=widgets.TimeWidget)
    intro_jingle = forms.ChoiceField(
        label=_('Intro Jingle'), choices=get_optional_jingle_choices, required=False
    )
    jingles = forms.MultipleChoiceField(
        label=_('Jingles'), widget=widgets.MulticheckWidget, required=False, choices=get_jingle_choices
    )

    weight_lang_en = forms.IntegerField(
        label=_('Weight adjustment for English tracks'),
        min_value=-10,
        max_value=10,
        widget=widgets.RangeWidget,
    )
    weight_lang_fr = forms.IntegerField(
        label=_('Weight adjustment for French tracks'),
        min_value=-10,
        max_value=10,
        widget=widgets.RangeWidget,
    )
    weight_lang_nl = forms.IntegerField(
        label=_('Weight adjustment for Dutch tracks'), min_value=-10, max_value=10, widget=widgets.RangeWidget
    )
    weight_lang_other = forms.IntegerField(
        label=_('Weight adjustment for tracks in other languages'),
        min_value=-10,
        max_value=10,
        widget=widgets.RangeWidget,
    )
    weight_instru = forms.IntegerField(
        label=_('Weight adjustment for instrumental tracks'),
        min_value=-10,
        max_value=10,
        widget=widgets.RangeWidget,
    )
    weight_cfwb = forms.IntegerField(
        label=_('Weight adjustment for FWB tracks'), min_value=-10, max_value=10, widget=widgets.RangeWidget
    )
    weight_newest = forms.IntegerField(
        label=_('Weight adjustement for newest tracks'),
        min_value=-10,
        max_value=10,
        widget=widgets.RangeWidget,
    )
    weight_new = forms.IntegerField(
        label=_('Weight adjustement for new tracks'), min_value=-10, max_value=10, widget=widgets.RangeWidget
    )

    extra_zones = forms.MultipleChoiceField(
        label=_('Extra zones'), widget=widgets.MulticheckWidget, required=False, choices=get_nonstop_zones
    )
