# proxy between arduino switch and websockets
# it listens on 0.0.0.0, UDP port 1312, "websocket" port 8765

import asyncio
import json
import logging

import requests
import websockets

from nonstop.app_settings import app_settings

logger = logging.getLogger('switch-proxy')


class PanikSwitchProtocol(asyncio.DatagramProtocol):
    UDP_PORT = 1312

    def __init__(self):
        super().__init__()
        self.currently_active = 0
        self.update_with_http()
        self.websocket_connections = set()

    def datagram_received(self, data, addr):
        logger.debug('Datagram received: %s', data)
        new_active = json.loads(data)['active']
        if new_active != self.currently_active:
            logger.info('UDP update, %s -> %s', self.currently_active, new_active)
        self.currently_active = json.loads(data)['active']

    async def websocket_handler(self, websocket, path):
        self.websocket_connections.add(websocket)
        latest_active = None
        while True:
            if self.currently_active != latest_active:
                latest_active = self.currently_active
                to_trash = []
                for ws in self.websocket_connections:
                    try:
                        await ws.send(json.dumps({'active': self.currently_active}))
                    except websockets.exceptions.ConnectionClosed:
                        to_trash.append(ws)
                for ws in to_trash:
                    self.websocket_connections.remove(ws)
            await asyncio.sleep(0.1)

    def update_with_http(self):
        try:
            resp = requests.get(app_settings.ON_AIR_SWITCH_URL, timeout=2)
            if resp.ok:
                new_active = resp.json().get('active')
                if new_active != self.currently_active:
                    logger.info('HTTP-only update, %s -> %s', self.currently_active, new_active)
                self.currently_active = resp.json().get('active')
        except (OSError, requests.exceptions.RequestException):
            pass


async def main():
    loop = asyncio.get_running_loop()
    proto = PanikSwitchProtocol()
    await websockets.serve(proto.websocket_handler, '0.0.0.0', 8765)
    transport, protocol = await loop.create_datagram_endpoint(lambda: proto, local_addr=('0.0.0.0', 1312))
    try:
        while True:
            await asyncio.sleep(300)
            proto.update_with_http()
    finally:
        transport.close()


asyncio.run(main())
