import datetime
import html
import logging
import time

import requests
from django.core.management.base import BaseCommand, CommandError
from django.utils.dateparse import parse_datetime

from nonstop.models import Artist, SomaLogLine, Track

logger = logging.getLogger('track_airtime_tracks')


def parse_duration(duration):
    # ex: "00:01:30.07025" -> 90.07025 (seconds)
    parts = [float(x) for x in duration.split(':')]
    return datetime.timedelta(seconds=parts[2] + (parts[1] * 60) + (parts[0] * 60 * 60))


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--live-info-url', help='https://foobar.airtime.pro/api/live-info-v2')

    def handle(self, verbosity, live_info_url, **options):
        if not live_info_url:
            raise CommandError('missing --live-info-url')

        # run until killed, and starts back where it was killed
        while True:
            logger.debug('Checking API')
            resp = requests.get(live_info_url)
            if not resp.ok:
                logger.error('Got invalid API answer (%s)', resp.status_code)
                time.sleep(60)
                continue
            if resp.json().get('shows', {}).get('current', {}).get('auto_dj'):
                # API returns metadata for previous/next track and a useless
                # "livestream" entry for the currently playing track; so we
                # look at them all.
                logger.debug('(autodj mode)')
                parts = ('previous', 'current', 'next')
            else:
                parts = ('current',)
            for part_name in parts:
                part = resp.json().get('tracks', {}).get(part_name)
                if part is None:
                    continue
                starts = part.get('starts')
                if part.get('type') == 'track':
                    metadata = part.get('metadata')
                    if metadata.get('artist_name'):
                        artist, created = Artist.objects.get_or_create(
                            name=html.unescape(metadata.get('artist_name'))
                        )
                    else:
                        artist = None
                    track, created = Track.objects.get_or_create(
                        artist=artist,
                        title=html.unescape(metadata.get('track_title')),
                        duration=parse_duration(metadata.get('length')),
                    )
                    if created:
                        logger.info('New track: %s', track)
                    logline, created = SomaLogLine.objects.get_or_create(
                        track=track, play_timestamp=parse_datetime(starts)
                    )
                    if created:
                        logger.info('Playing at %s: %s', starts, track)

            if not resp.json().get('tracks', {}).get('next'):
                logger.warning('No known next track')
                time.sleep(30)
                continue

            next_datetime = parse_datetime(resp.json().get('tracks', {}).get('next', {}).get('starts'))
            wait_until = min(datetime.datetime.now() + datetime.timedelta(minutes=5), next_datetime)
            waiting_time = (wait_until - datetime.datetime.now()).total_seconds()
            logger.debug('Waiting for %d seconds (announced change is %s)', waiting_time, next_datetime)
            time.sleep(waiting_time + 1)  # wait at least 1 second
